package com.cedarsoft.business.contact;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

/**
 * Represents an address.
 */
public class Address extends BaseAddress {
  private static final long serialVersionUID = -4770555205554903188L;

  @NonNls
  @NotNull
  public static final String PROPERTY_STREET = "street";

  @NotNull
  @NonNls
  private String street = "";

  public Address() {
  }

  @Override
  public void setAddressLine( @NotNull @NonNls String addressLine ) {
    setStreet( addressLine );
  }

  @Override
  @NotNull
  @NonNls
  public String getAddressLine() {
    return getStreet();
  }

  public Address( @NonNls @NotNull String street, @NotNull City city, @NotNull Country country ) {
    this( street, city, country, Classification.PRIVATE );
  }

  public Address( @NonNls @NotNull String street, @NotNull City city, @NotNull Country country, @NotNull Classification classification ) {
    this( street, city, country, classification, false );
  }

  public Address( @NonNls @NotNull String street, @NotNull City city, @NotNull Country country, @NotNull Classification classification, boolean active ) {
    super( city, country, classification, active );
    this.street = street;
  }

  @NotNull
  public String getStreet() {
    lock.readLock().lock();
    try {
      return street;
    } finally {
      lock.readLock().unlock();
    }
  }

  public void setStreet( @NonNls @NotNull String street ) {
    lock.writeLock().lock();
    try {
      pcs.firePropertyChange( PROPERTY_STREET, this.street, this.street = street );
    } finally {
      lock.writeLock().unlock();
    }
  }

  @Override
  public String toString() {
    lock.readLock().lock();
    try {
      return street + ", " + city + " (" + country + ')';
    } finally {
      lock.readLock().unlock();
    }
  }
}
