package com.cedarsoft.business.contact;

import com.cedarsoft.business.contact.communication.AbstractCommunicationChannel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 *
 */
public abstract class BaseAddress extends AbstractCommunicationChannel {
  @NonNls
  @NotNull
  public static final String PROPERTY_CITY = "city";
  @NonNls
  @NotNull
  public static final String PROPERTY_COUNTRY = "country";
  @NonNls
  @NotNull
  public static final String PROPERTY_ADDRESS_LINE = "addressLine";

  @NotNull
  protected City city = new City( "", "" );
  @NotNull
  protected Country country = Country.Germany;

  protected BaseAddress() {
  }

  protected BaseAddress( @NotNull City city, @NotNull Country country, @NotNull Classification classification, boolean active ) {
    super( classification, active );
    this.city = city;
    this.country = country;
  }

  @NotNull
  public City getCity() {
    lock.readLock().lock();
    try {
      return city;
    } finally {
      lock.readLock().unlock();
    }
  }

  public void setCity( @NotNull City city ) {
    lock.writeLock().lock();
    try {
      pcs.firePropertyChange( PROPERTY_CITY, this.city, this.city = city );
    } finally {
      lock.writeLock().unlock();
    }
  }

  @NotNull
  public Country getCountry() {
    lock.readLock().lock();
    try {
      return country;
    } finally {
      lock.readLock().unlock();
    }
  }

  public void setCountry( @NotNull Country country ) {
    lock.writeLock().lock();
    try {
      pcs.firePropertyChange( PROPERTY_COUNTRY, this.country, this.country = country );
    } finally {
      lock.writeLock().unlock();
    }
  }

  @NotNull
  @NonNls
  public String getRepresentation() {
    lock.readLock().lock();
    try {
      return getAddressLine() + ", " + getCity() + " (" + getCountry() + ')';
    } finally {
      lock.readLock().unlock();
    }
  }

  /**
   * Sets the address line
   *
   * @param addressLine the address line
   */
  public abstract void setAddressLine( @NotNull @NonNls String addressLine );

  @NotNull
  @NonNls
  public abstract String getAddressLine();

  public static boolean hasSameRepresentation( @Nullable BaseAddress first, @Nullable BaseAddress other ) {
    //noinspection ObjectEquality
    if ( first == other ) {
      return true;
    }

    if ( first != null ) {
      if ( other == null ) {
        return false;
      } else {
        return first.getRepresentation().equals( other.getRepresentation() );
      }
    } else {
      return false;
    }
  }
}
