package com.cedarsoft.business.contact;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

/**
 *
 */
public class POBAddress extends BaseAddress {
  @NonNls
  @NotNull
  public static final String PROPERTY_NUMBER = "number";

  @NotNull
  @NonNls
  private String number = "";

  public POBAddress() {
  }

  public POBAddress( @NonNls @NotNull String street, @NotNull City city, @NotNull Country country ) {
    this( street, city, country, Classification.PRIVATE );
  }

  public POBAddress( @NotNull String number, @NotNull City city, @NotNull Country country, @NotNull Classification classification ) {
    this( number, city, country, classification, false );
  }

  public POBAddress( @NotNull String number, @NotNull City city, @NotNull Country country, @NotNull Classification classification, boolean active ) {
    super( city, country, classification, active );
    this.number = number;
  }

  @NotNull
  @NonNls
  public String getNumber() {
    lock.readLock().lock();
    try {
      return number;
    } finally {
      lock.readLock().unlock();
    }
  }

  public void setNumber( @NotNull @NonNls String number ) {
    lock.writeLock().lock();
    try {
      pcs.firePropertyChange( PROPERTY_NUMBER, this.number, this.number = number );
    } finally {
      lock.writeLock().unlock();
    }
  }

  @Override
  public void setAddressLine( @NotNull @NonNls String addressLine ) {
    setNumber( addressLine );
  }

  @Override
  @NotNull
  @NonNls
  public String getAddressLine() {
    return getNumber();
  }

  @Override
  public String toString() {
    lock.readLock().lock();
    try {
      return number + ", " + city + " (" + country + ')';
    } finally {
      lock.readLock().unlock();
    }
  }
}
