package com.cedarsoft.business.contact.communication;

import com.cedarsoft.business.contact.Classification;
import com.cedarsoft.utils.event.ClusteredPropertyChangeSupport;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import java.beans.PropertyChangeListener;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 *
 */
public abstract class AbstractCommunicationChannel implements CommunicationChannel, Comparable<CommunicationChannel> {
  @NotNull
  protected ReadWriteLock lock = new ReentrantReadWriteLock();

  @NotNull
  protected final ClusteredPropertyChangeSupport pcs = new ClusteredPropertyChangeSupport( this );

  private Long id;
  @NonNls
  protected Classification classification = Classification.PRIVATE;
  @NotNull
  protected String description = "";

  protected boolean active;

  protected AbstractCommunicationChannel() {
    this( Classification.PRIVATE, false );
  }

  protected AbstractCommunicationChannel( @NotNull Classification classification, boolean active ) {
    this.classification = classification;
    this.active = active;
  }

  @NotNull
  public Classification getClassification() {
    lock.readLock().lock();
    try {
      return classification;
    } finally {
      lock.readLock().unlock();
    }
  }

  public void setClassification( @NotNull Classification classification ) {
    lock.writeLock().lock();
    try {
      pcs.firePropertyChange( PROPERTY_CLASSIFICATION, this.classification, this.classification = classification );
    } finally {
      lock.writeLock().unlock();
    }
  }

  @NotNull
  public String getDescription() {
    lock.readLock().lock();
    try {
      return description;
    } finally {
      lock.readLock().unlock();
    }
  }

  public void setDescription( @NotNull String description ) {
    lock.writeLock().lock();
    try {
      pcs.firePropertyChange( PROPERTY_DESCRIPTION, this.description, this.description = description );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public boolean isActive() {
    lock.readLock().lock();
    try {
      return active;
    } finally {
      lock.readLock().unlock();
    }
  }

  public void setActive( boolean active ) {
    lock.writeLock().lock();
    try {
      pcs.firePropertyChange( PROPERTY_ACTIVE, this.active, this.active = active );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void removePropertyChangeListener( @NotNull PropertyChangeListener listener ) {
    lock.writeLock().lock();
    try {
      pcs.removePropertyChangeListener( listener );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void removePropertyChangeListener( @NotNull @NonNls String propertyName, @NotNull PropertyChangeListener listener ) {
    lock.writeLock().lock();
    try {
      pcs.removePropertyChangeListener( propertyName, listener );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void addPropertyChangeListener( @NotNull PropertyChangeListener listener ) {
    lock.writeLock().lock();
    try {
      pcs.addPropertyChangeListener( listener );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void addPropertyChangeListener( @NotNull PropertyChangeListener listener, boolean isTransient ) {
    lock.writeLock().lock();
    try {
      pcs.addPropertyChangeListener( listener, isTransient );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void addPropertyChangeListener( @NotNull @NonNls String propertyName, @NotNull PropertyChangeListener listener ) {
    lock.writeLock().lock();
    try {
      pcs.addPropertyChangeListener( propertyName, listener );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void addPropertyChangeListener( @NotNull @NonNls String propertyName, @NotNull PropertyChangeListener listener, boolean isTransient ) {
    lock.writeLock().lock();
    try {
      pcs.addPropertyChangeListener( propertyName, listener, isTransient );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public int compareTo( CommunicationChannel o ) {
    lock.readLock().lock();
    try {
      return toString().compareTo( o.toString() );
    } finally {
      lock.readLock().unlock();
    }
  }
}
