package com.cedarsoft.business.contact.communication;

import com.cedarsoft.business.contact.Classification;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

/**
 *
 */
public abstract class NumberBasedCommunicationChannel extends AbstractCommunicationChannel {
  @NonNls
  public static final String PROPERTY_AREA_CODE = "areaCode";
  @NonNls
  public static final String PROPERTY_COUNTRY_CODE = "countryCode";
  @NonNls
  public static final String PROPERTY_NUMBER = "number";

  @NotNull
  @NonNls
  protected String areaCode = "";
  @NotNull
  @NonNls
  protected String countryCode = "";
  @NotNull
  @NonNls
  protected String number = "";

  /**
   * Hibernate
   */
  @Deprecated
  protected NumberBasedCommunicationChannel() {
  }

  public NumberBasedCommunicationChannel( @NonNls @NotNull String countryCode, @NotNull @NonNls String areaCode, @NotNull @NonNls String number ) {
    this.areaCode = areaCode;
    this.countryCode = countryCode;
    this.number = number;
  }

  protected NumberBasedCommunicationChannel( @NonNls @NotNull String countryCode, @NotNull @NonNls String areaCode, @NotNull @NonNls String number, Classification classification ) {
    this.countryCode = countryCode;
    this.areaCode = areaCode;
    this.number = number;
    this.classification = classification;
  }

  @NotNull
  @NonNls
  public String getAreaCode() {
    lock.readLock().lock();
    try {
      return areaCode;
    } finally {
      lock.readLock().unlock();
    }
  }

  @NotNull
  @NonNls
  public String getCountryCode() {
    lock.readLock().lock();
    try {
      return countryCode;
    } finally {
      lock.readLock().unlock();
    }
  }

  @NotNull
  @NonNls
  public String getNumber() {
    lock.readLock().lock();
    try {
      return number;
    } finally {
      lock.readLock().unlock();
    }
  }

  public void setAreaCode( @NonNls @NotNull String areaCode ) {
    lock.writeLock().lock();
    try {
      pcs.firePropertyChange( PROPERTY_AREA_CODE, this.areaCode, this.areaCode = areaCode );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void setCountryCode( @NonNls @NotNull String countryCode ) {
    lock.writeLock().lock();
    try {
      pcs.firePropertyChange( PROPERTY_COUNTRY_CODE, this.countryCode, this.countryCode = countryCode );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void setNumber( @NotNull @NonNls String number ) {
    lock.writeLock().lock();
    try {
      pcs.firePropertyChange( PROPERTY_NUMBER, this.number, this.number = number );
    } finally {
      lock.writeLock().unlock();
    }
  }

  @NotNull
  @NonNls
  public String getRepresentation() {
    return getCountryCode() + ' ' + getAreaCode() + ' ' + getNumber();
  }
}
